@@ -2,11 +2,13 @@ from . import *
 
 
 class CookBookUi:
-	def list_uids_names(self):
+	def list_uids_names(self,assistMsg=''):
+		if assistMsg != '':
+			assistMsg+="\n"
 		while True:
 			try:
 				with Session(ENGINE) as session:
-					stext=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Name or UID?[b=skip]",helpText="b|back skips",data="string")
+					stext=Prompt.__init2__(None,func=FormBuilderMkText,ptext=f"{Fore.orange_red_1}{assistMsg}{Fore.light_yellow}Name or UID?[b=skip]",helpText="b|back skips",data="string")
 					if stext is None:
 						return None,None
 					if stext in ['d',]:
@@ -95,7 +97,8 @@ class CookBookUi:
 		#edit everything found by searchtext using recipe_uid as the final selection parameter
 	def create_new_rcp(self):
 		with Session(ENGINE) as session:
-			ruid,rname=self.list_uids_names()
+			assistMsg=f"search for existing recipe to add to..."
+			ruid,rname=self.list_uids_names(assistMsg=assistMsg)
 			uid=uuid1()
 			uid=str(uid)
 			excludes=['cbid','DTOE','recipe_uid','Instructions','recipe_name']
