@@ -1,5 +1,7 @@
 from . import *
 import uuid
+import radboy.TasksMode as TM
+
 class BhTrSa_Gui:
 
 	def next_saa(self):
@@ -519,6 +521,11 @@ class BhTrSa_Gui:
 			'exec':self.next_saa,
 			'desc':"display next x of schedules and appointments entry that are not past due"
 			},
+			uuid.uuid1():{
+			'cmds':["#"+str(0),*[i for i in generate_cmds(startcmd=["phonebook","phnbk"],endCmd=["",])]],
+			'exec':lambda self=self:TM.Tasks.TasksMode(parent=self,engine=db.ENGINE,init_only=True).phonebook(),
+			'desc':"open phonebook menu"
+			},
 		}
 		htext=[]
 		for cmd in cmds:
