@@ -30,6 +30,128 @@ from copy import copy
 from radboy.DB.OrderedAndRxd import *
 
 class HealthLogUi:
+	def last_dose(self):
+		with localcontext() as ctx:
+			with Session(ENGINE) as session:
+				status=0
+				ct=5
+				
+				lai=session.query(HealthLog).filter(and_(HealthLog.LongActingInsulinName!=None,HealthLog.LongActingInsulinTaken!=None))
+				lai=orderQuery(lai,HealthLog.DTOE,inverse=True).first()
+				if lai:
+					print("Long Acting Insulin")
+					num=0
+					view=[]
+					for x in lai.__table__.columns:
+						if getattr(lai,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(lai,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				sai=session.query(HealthLog).filter(and_(HealthLog.ShortActingInsulinName!=None,HealthLog.ShortActingInsulinTaken!=None))
+				sai=orderQuery(sai,HealthLog.DTOE,inverse=True).first()
+				if sai:
+					print("Short Acting Insulin")
+					num=1
+					view=[]
+					for x in sai.__table__.columns:
+						if getattr(sai,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(sai,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				last_carb=session.query(HealthLog).filter(and_(HealthLog.CarboHydrateIntake!=None,HealthLog.EntryName!=None))
+				last_carb=orderQuery(last_carb,HealthLog.DTOE,inverse=True).first()
+				if last_carb:
+					Print("CarboHydrateIntake")
+					num=2					
+					view=[]
+					for x in last_carb.__table__.columns:
+						if getattr(last_carb,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(last_carb,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				drug=session.query(HealthLog).filter(HealthLog.DrugQtyConsumed!=None)
+				drug=orderQuery(drug,HealthLog.DTOE,inverse=True).first()
+				if drug:
+					print("Last Drug Dose Taken")
+					num=3
+					view=[]
+					for x in drug.__table__.columns:
+						if getattr(drug,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(drug,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+
+				glucose=session.query(HealthLog).filter(and_(HealthLog.BloodSugar!=None,))
+				glucose=orderQuery(glucose,HealthLog.DTOE,inverse=True).first()
+				if glucose:
+					print("Blood Sugar")
+					num=0
+					view=[]
+					for x in glucose.__table__.columns:
+						if getattr(glucose,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(glucose,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				height=session.query(HealthLog).filter(and_(HealthLog.BloodSugar!=None,))
+				height=orderQuery(height,HealthLog.DTOE,inverse=True).first()
+				if height:
+					print("Height")
+					num=0
+					view=[]
+					for x in height.__table__.columns:
+						if getattr(height,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(height,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				weight=session.query(HealthLog).filter(and_(HealthLog.BloodSugar!=None,))
+				weight=orderQuery(weight,HealthLog.DTOE,inverse=True).first()
+				if weight:
+					print("Weight")
+					num=0
+					view=[]
+					for x in weight.__table__.columns:
+						if getattr(weight,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(weight,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				heartRate=session.query(HealthLog).filter(and_(HealthLog.BloodSugar!=None,))
+				heartRate=orderQuery(heartRate,HealthLog.DTOE,inverse=True).first()
+				if heartRate:
+					print("Heart Rate")
+					num=0
+					view=[]
+					for x in heartRate.__table__.columns:
+						if getattr(heartRate,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(heartRate,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+				else:
+					status+=1
+
+				if status >= 4:
+					print("There is nothing to display!")
+				
 	def edit_hlid(self):
 		with Session(ENGINE) as session:
 			hlid=Prompt.__init2__(None,func=FormBuilderMkText,ptext="HLID to Edit?: ",helpText="what healthlod id do you wish to edit?",data="integer")
@@ -57,6 +179,127 @@ class HealthLogUi:
 					session.refresh(hl)
 					print(std_colorize(f"Updated {hl}",0,1))
 
+	def search(self):
+		page=Prompt.__init2__(None,func=FormBuilderMkText,ptext="page?",helpText="page/display one at a time y/n",data="boolean")
+		if page in ['None',]:
+			return None
+		elif page in [True,'d']:
+			page=True
+		with localcontext() as CTX:
+			with Session(ENGINE) as session:
+				fields={
+				i.name:{
+				'default':None,
+				'type':str(i.type).lower()
+				} for i in HealthLog.__table__.columns
+				}
+				fd=FormBuilder(data=fields,passThruText="Your Search Terms `showkeys` and `gotoi` are useful.")
+				if fd is None:
+					return
+				FD={}
+				for i in fd:
+					if fd[i] is not None:
+						FD[i]=fd[i]
+				fd=FD
+				filte=[]
+				for i in fd:
+					if isinstance(fd[i],str):
+						filte.append(getattr(HealthLog,i).icontains(fd[i]))
+					else:
+						filte.append(getattr(HealthLog,i)==fd[i])
+				query=session.query(HealthLog).filter(and_(*filte))
+				ordered=orderQuery(query,HealthLog.DTOE)
+				lo={
+				'limit':{
+				'default':10,
+				'type':'integer',
+				},
+				'offset':{
+				'type':'integer',
+				'default':0,
+				}
+				}
+				lod=FormBuilder(data=lo,passThruText="Limit your results?")
+				if lod is None:
+					return
+
+				loq=limitOffset(query,lod['limit'],lod['offset'])
+				results=loq.all()
+				ct=len(results)
+				if len(results) < 1:
+					print("No Results!")
+					return
+				for num,i in enumerate(results):
+					view=[]
+					for x in i.__table__.columns:
+						if getattr(i,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(i,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+					if page:
+						n=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Next?",helpText="next, anything else is goes next, just b/q that diff",data="boolean")
+						if n in ['NaN',None]:
+							return None
+						else:
+							continue
+
+
+	def searchText(self):
+		page=Prompt.__init2__(None,func=FormBuilderMkText,ptext="page?",helpText="page/display one at a time y/n",data="boolean")
+		if page in ['None',]:
+			return None
+		elif page in [True,'d']:
+			page=True
+		filte=[]
+		with localcontext() as CTX:
+			with Session(ENGINE) as session:
+				query=session.query(HealthLog)
+				fields=[i for i in HealthLog.__table__.columns if str(i.type).lower() in ['string','varchar','char','text']]
+				
+				term=Control(func=FormBuilderMkText,ptext="What are you searching?",helpText="just text is searched.",data="string")
+				if term in [None,'NaN','d']:
+					return
+				else:
+					filte=[]
+					for i in fields:
+						filte.append((getattr(HealthLog,i.name).icontains(term)))
+
+				query=session.query(HealthLog).filter(or_(*filte))
+				ordered=orderQuery(query,HealthLog.DTOE)
+				lo={
+				'limit':{
+				'default':10,
+				'type':'integer',
+				},
+				'offset':{
+				'type':'integer',
+				'default':0,
+				}
+				}
+				lod=FormBuilder(data=lo,passThruText="Limit your results?")
+				if lod is None:
+					return
+
+				loq=limitOffset(query,lod['limit'],lod['offset'])
+				results=loq.all()
+				ct=len(results)
+				if len(results) < 1:
+					print("No Results!")
+					return
+				for num,i in enumerate(results):
+					view=[]
+					for x in i.__table__.columns:
+						if getattr(i,str(x.name)) not in [None]:
+							view.append(f'{Fore.green_3b}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={Fore.sea_green_2}{str(getattr(i,str(x.name)))}{Style.reset}')
+					msg=f"{'|'.join(view)}"
+					print(std_colorize(msg,num,ct))
+					if page:
+						n=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Next?",helpText="next, anything else is goes next, just b/q that diff",data="boolean")
+						if n in ['NaN',None]:
+							return None
+						else:
+							continue
+
 	def new_health_log(self):
 		with Session(ENGINE) as session:
 			hl=HealthLog()
@@ -565,13 +808,18 @@ class HealthLogUi:
 						pass
 			print(f"{Fore.light_magenta}Dates on the Graph(s) are in the format of {Fore.orange_red_1}Day/Month/Year{Fore.light_magenta}, whereas Date Input will remain {Fore.light_steel_blue}Month/Day/Year{Style.reset}")
 			n=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Next?",helpText="next y/n",data="boolean")
-			if n in ['None',]:
+			if n in ['NaN',None]:
 				return None
 			elif n in [True,]:
 				return True
 
 
 	def showAllField(self,fields=[],not_none=[],total_drg=False,total_fd=False):
+		page=Prompt.__init2__(None,func=FormBuilderMkText,ptext="page?",helpText="page/display one at a time y/n",data="boolean")
+		if page in ['None',]:
+			return None
+		elif page in [True,'d']:
+			page=True
 		unit_registry=pint.UnitRegistry()
 		try:
 			useDateRange=Prompt.__init2__(None,func=FormBuilderMkText,ptext="use a date range?",helpText="yes or no",data="boolean")
@@ -682,6 +930,12 @@ class HealthLogUi:
 						view.append(f'{color_date}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={color}{str(getattr(i,str(x.name)))}{Style.reset}')
 					msg=f"{Fore.light_green}{num}{Fore.light_yellow}/{num+1} of {Fore.light_red}{ct} ->{'|'.join(view)}"
 					print(msg)
+					if page:
+						n=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Next?",helpText="next, anything else is goes next, just b/q that diff",data="boolean")
+						if n in ['NaN',None]:
+							return None
+						else:
+							continue
 				if total_drg or total_fd:
 					print(f"{Fore.orange_red_1}Broken into Totals{Style.reset}")
 					ct=len(dummy)
@@ -699,6 +953,12 @@ class HealthLogUi:
 							view.append(f'{color_date}{Style.bold}{str(x.name)}{Fore.deep_sky_blue_1}={color}{str(getattr(i,str(x.name)))}{Style.reset}')
 						msg=f"{Fore.light_green}{num}{Fore.light_yellow}/{num+1} of {Fore.light_red}{ct} ->{'|'.join(view)}"
 						print(msg)
+						if page:
+							n=Prompt.__init2__(None,func=FormBuilderMkText,ptext="Next?",helpText="next, anything else is goes next, just b/q that diff",data="boolean")
+							if n in ['NaN',None]:
+								return None
+							else:
+								continue
 		except Exception as e:
 			print(e)
 
@@ -919,10 +1179,25 @@ class HealthLogUi:
 			'exec':lambda self=self:self.export_log_field(fields=['BloodSugar','BloodSugarUnitName','LongActingInsulinName','LongActingInsulinTaken','LongActingInsulinUnitName','ShortActingInsulinName','ShortActingInsulinTaken','ShortActingInsulinUnitName','HeartRate','HeartRateUnitName','DrugConsumed','DrugQtyConsumed','DrugQtyConsumedUnitName','CarboHydrateIntake','CarboHydrateIntakeUnitName','Comments',],not_none=['Comments',])
 			},
 			str(uuid1()):{
+			'cmds':generate_cmds(startcmd=['sch','search'],endCmd=['specific','spec','spcfc','finer']),
+			'desc':'search with formbuilder with just equals/icontains',
+			'exec':lambda self=self:self.search()
+			},
+			str(uuid1()):{
+			'cmds':generate_cmds(startcmd=['sch','search'],endCmd=['text','txt','str','t']),
+			'desc':'search with text only fields by term',
+			'exec':lambda self=self:self.searchText()
+			},
+			str(uuid1()):{
 			'cmds':['ordered and recieved','oar','ordered and rxd','ordered & rxd','ordered & recieved','o&r'],
 			'desc':'ordered and recieved journal',
 			'exec':lambda self=self:OrderAndRxdUi()
 			},
+			str(uuid1()):{
+			'cmds':generate_cmds(startcmd=['last',],endCmd=['dse','dose','ds']),
+			'desc':'review last doses',
+			'exec':lambda self=self:self.last_dose()
+			},
 			'xpt consumed':{
 				'cmds':['xptfd','xpt fd','xpt food','xpt-fd','xpt fuel','xpt fl','xlfl'],
 				'desc':'export food',
